/*========================================================================
  Function Viewer: visualizer module
  ---------------
  This program is distributed under the GPL license.

  Creates and takes care of updating the function drawing area.
==========================================================================*/

#ifndef FUNCVIEWER_VISUALIZER_HH
#define FUNCVIEWER_VISUALIZER_HH

#include "Function.hh"

#include <list>
#include <gtk/gtk.h>

class Visualizer: public Function::Callback
{
 public:
    Visualizer(GtkWidget* parent, GtkWidget* buttonBar);
    virtual ~Visualizer();

    void addFunction(Function*);

    // From Function::Callback:
    virtual void functionHasChanged();
    virtual void functionIsBeingDestroyed(Function*);

    inline GtkWidget* getWidget();



//----------------------------------------------------------------------
 private:
//----------------------------------------------------------------------
    GtkWidget* drawingArea;
    GdkPixmap* pixMap;
    GdkGC* drawingGC;
    GdkColormap* colorMap;
    PangoLayout* pLayout;

    std::list<Function*> functions;

    int width, height, prevMouseX, prevMouseY;
    double centerX, centerY, horizontalSize, aspectRatio;

    void draw();
    inline void setColor(unsigned red, unsigned green, unsigned blue);
    inline void refreshArea(int x, int y, int w, int h);
    inline void horizontalLine(int y);
    inline void verticalLine(int x);
    void drawAxes();
    void drawFunction(Function*);

    template<typename CBFunc>
    void Visualizer::createButton(const char*, CBFunc, GtkWidget*);

    static gboolean configureCallback(GtkWidget*, GdkEventConfigure*,
                                      gpointer);
    static gboolean exposeCallback(GtkWidget*, GdkEventExpose*, gpointer);
    static gboolean mouseClickCallback(GtkWidget*, GdkEventButton*, gpointer);
    static gboolean mouseMotionCallback(GtkWidget*, GdkEventMotion*, gpointer);
    static void resetScrollCallback(GtkToolButton*, gpointer);
    static void resetZoomCallback(GtkToolButton*, gpointer);
    static void resetAspectRatioCallback(GtkToolButton*, gpointer);

    Visualizer(const Visualizer&);
    Visualizer& operator=(const Visualizer&);
};

inline GtkWidget* Visualizer::getWidget() { return drawingArea; }

#endif
